"use strict";

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var Url = require('url-parse');

var cookie = require('cookie');

var safeStringify = require('fast-safe-stringify'); // <https://lacke.mn/reduce-your-bundle-js-file-size/>
// <https://github.com/lodash/babel-plugin-lodash/issues/221>


var pick = require('lodash/pick');

var isString = require('lodash/isString');

var isObject = require('lodash/isObject');

var clone = require('lodash/clone');

var cloneDeep = require('lodash/cloneDeep');

var isUndefined = require('lodash/isUndefined');

var isNull = require('lodash/isNull');

var isFunction = require('lodash/isFunction');

var isEmpty = require('lodash/isEmpty');

var isFinite = require('lodash/isFinite');

var isArray = require('lodash/isArray');

var mapValues = require('lodash/mapValues');

var creditCardType = require('credit-card-type');

var sensitiveFields = require('sensitive-fields'); // <https://github.com/braintree/credit-card-type/issues/90>


function isCreditCard(val) {
  var digits = val.replace(/\D/g, '');
  var types = creditCardType(digits);
  if (!isArray(types) || isEmpty(types)) return false;
  var match = false;

  for (var t = 0; t < types.length; t++) {
    if (match) break;
    var type = types[t]; // can match any one of the lengths

    if (!isArray(type.lengths) || isEmpty(type.lengths)) continue;

    for (var l = 0; l < type.lengths.length; l++) {
      var len = type.lengths[l];

      if (isFinite(len) && len === digits.length) {
        match = true;
        break;
      }
    }
  }

  return match;
} // https://stackoverflow.com/a/39087474


function maskProps(obj, props) {
  var maskCreditCards = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
  var isHeaders = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;
  return mapValues(obj, function (val, key) {
    if (isObject(val)) return maskProps(val, props, maskCreditCards, isHeaders);
    if (!isString(val)) return val; // if it was a credit card then replace all digits with asterisk

    if (!isHeaders && maskCreditCards && isCreditCard(val)) return val.replace(/[^\D\s]/g, '*');
    if (!props.includes(key)) return val; // replace only the authentication <credentials> portion with asterisk
    // Authorization: <type> <credentials>

    if (isHeaders && key === 'Authorization') return "".concat(val.split(' ')[0], " ").concat(val.substring(val.indexOf(' ') + 1).replace(/./g, '*'));
    return val.replace(/./g, '*');
  });
} // inspired by raven's parseRequest
// eslint-disable-next-line complexity


var parseRequest = function parseRequest() {
  var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  config = _objectSpread({
    req: {},
    userFields: ['id', 'email', 'full_name', 'ip_address'],
    sanitizeFields: sensitiveFields,
    sanitizeHeaders: ['Authorization'],
    maskCreditCards: true
  }, config);
  var _config = config,
      req = _config.req,
      userFields = _config.userFields,
      sanitizeFields = _config.sanitizeFields,
      sanitizeHeaders = _config.sanitizeHeaders,
      maskCreditCards = _config.maskCreditCards;
  var headers = maskProps(req.headers || req.header || {}, sanitizeHeaders, maskCreditCards, true);
  var method = req.method || 'GET'; // inspired from `preserve-qs` package

  var originalUrl = '';

  if (isString(req.originalUrl)) {
    originalUrl = req.originalUrl;
  } else if (isString(req.url)) originalUrl = req.url;else if (process.browser) originalUrl = window.location.pathname + window.location.search;

  originalUrl = new Url(originalUrl); // parse query, path, and origin to prepare absolute Url

  var query = Url.qs.parse(originalUrl.query);
  var path = originalUrl.origin === 'null' ? originalUrl.pathname : "".concat(originalUrl.origin).concat(originalUrl.pathname);
  var qs = Url.qs.stringify(query, true);
  var absoluteUrl = path + qs; // default to the user object

  var user = isObject(req.user) ? isFunction(req.user.toObject) ? req.user.toObject() : clone(req.user) : {};
  var ip = '';

  if (isString(req.ip)) {
    ip = req.ip;
  } else if (isObject(req.connection) && isString(req.connection.remoteAddress)) ip = req.connection.remoteAddress;

  if (ip && !isString(user.ip_address)) user.ip_address = ip;
  if (isArray(userFields) && !isEmpty(userFields)) user = pick(user, userFields); // recursively search through user and filter out passwords from it

  user = maskProps(user, sanitizeFields, maskCreditCards);
  var body = '';
  var originalBody = req._originalBody || req.body;
  if (!['GET', 'HEAD'].includes(method) && !isUndefined(originalBody)) body = isString(originalBody) ? clone(originalBody) : cloneDeep(originalBody); // recursively search through body and filter out passwords from it

  if (isObject(body)) body = maskProps(body, sanitizeFields, maskCreditCards);
  if (!isUndefined(body) && !isNull(body) && !isString(body)) body = safeStringify(body); // populate user agent and referrer if
  // we're in a browser and they're unset

  if (process.browser) {
    // set user agent
    if (typeof window.navigator !== 'undefined' && isObject(window.navigator) && isString(window.navigator.userAgent) && (!isString(headers['user-agent']) || !headers['user-agent'])) headers['user-agent'] = window.navigator.userAgent;

    if (typeof window.document !== 'undefined' && isObject(window.document)) {
      // set referrer
      if (isString(window.document.referrer) && (!isString(headers.referer) || !headers.referer) && (!isString(headers.referrer) || !headers.referrer)) headers.referer = window.document.referrer; // set cookie

      if (isString(window.document.cookie) && (!isString(headers.cookie) || !headers.cookie)) headers.cookie = window.document.cookie;
    }
  } // parse the cookies (if any were set)


  var cookies = cookie.parse(headers.cookie || '');
  return {
    request: {
      method: method,
      query: query,
      headers: headers,
      cookies: cookies,
      body: body,
      url: absoluteUrl
    },
    user: user
  };
};

module.exports = parseRequest;